// Cg_Shader.cpp: Implementierung der Klasse Cg_Shader
//
///////////////////////////////////////////////////////////////////////

#include <iostream>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <GL/glut.h>
#include "Cg_Shader.h"

using namespace std;

PFNGLMULTITEXCOORD1FARBPROC glMultiTexCoord1fARB = NULL;
PFNGLMULTITEXCOORD2FARBPROC glMultiTexCoord2fARB = NULL;
PFNGLMULTITEXCOORD3FARBPROC glMultiTexCoord3fARB = NULL;
PFNGLMULTITEXCOORD4FARBPROC glMultiTexCoord4fARB = NULL;

//new code
PFNGLTEXIMAGE3DPROC glTexImage3D = NULL;

PFNGLACTIVETEXTUREARBPROC glActiveTextureARB = NULL;
PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB = NULL;

PFNGLSECONDARYCOLOR3FEXTPROC glSecondaryColor3fEXT = NULL;
PFNGLSECONDARYCOLOR3FVEXTPROC glSecondaryColor3fvEXT = NULL;


void cgErrorCallback()
{
    CGerror err = cgGetError();

    if (err != CG_NO_ERROR) {
        cerr << "cgErrorCallback(): " << cgGetErrorString(err) << endl;
        exit(1);

    }
}


void InitializeCgFragment(char* fragmentShader, CGcontext& context, CGprofile& fragmentProfile, CGprogram &fragmentProgram)
{
	// define callback function for Cg errors
	cgSetErrorCallback(cgErrorCallback);
	// create a Cg context which will hold our shader programs
	cgSetErrorCallback(cgErrorCallback);
	context = cgCreateContext();

	// get latest profile supported by installed HW
	fragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	cgGLSetOptimalOptions(fragmentProfile);
	if (fragmentProfile == CG_PROFILE_UNKNOWN)
	{
		cout << "Graphics card not supported; could not find a suitable fragment shader profile.\n";
	}

	// create actual Cg program from file; last two parameters are entry point and arguments
	fragmentProgram = cgCreateProgramFromFile(context, CG_SOURCE, fragmentShader, fragmentProfile, NULL, NULL);
	if(!fragmentProgram)
	{
		cout << "Could not read and creaet fragment shader program.\n";
	}
    cgGLLoadProgram(fragmentProgram);


    //cgGLBindProgram(fragmentProgram);
    //cgGLEnableProfile(fragmentProfile);

}

void InitializeCgVertex(char* vertexShader, CGcontext& context, CGprofile& vertexProfile, CGprogram &vertexProgram)
{
	// define callback function for Cg errors
	cgSetErrorCallback(cgErrorCallback);
	// create a Cg context which will hold our shader programs
	cgSetErrorCallback(cgErrorCallback);
	context = cgCreateContext();

	// get latest profile supported by installed HW
	vertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	cgGLSetOptimalOptions(vertexProfile);
	if (vertexProfile == CG_PROFILE_UNKNOWN)
	{
		cout << "Graphics card not supported; could not find a suitable vertex shader profile.\n";
	}

	// create actual Cg program from file; last two parameters are entry point and arguments
	vertexProgram = cgCreateProgramFromFile(context, CG_SOURCE, vertexShader, vertexProfile, NULL, NULL);
	if(!vertexProgram)
	{
		cout << "Could not read and creaet vertex shader program.\n";
	}
    cgGLLoadProgram(vertexProgram);


    //cgGLBindProgram(fragmentProgram);
    //cgGLEnableProfile(fragmentProfile);

}

void InitializeCg(char* vertexShader, char* fragmentShader, CGcontext& context, CGprofile& vertexProfile, CGprofile& fragmentProfile, CGprogram &vertexProgram, CGprogram &fragmentProgram)
{

	// define callback function for Cg errors
	cgSetErrorCallback(cgErrorCallback);
	// create a Cg context which will hold our shader programs
	context = cgCreateContext();

	// get latest vertex shader profile supported by the installed HW
	vertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	cgGLSetOptimalOptions(vertexProfile);
	if (vertexProfile == CG_PROFILE_UNKNOWN)
	{
		cout << "Graphics card not supported; could not find a suitable vertex shader profile.\n";
	}

	// get latest fragment shader profile supported by the installed HW
	fragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	cgGLSetOptimalOptions(fragmentProfile);
	if (fragmentProfile == CG_PROFILE_UNKNOWN)
	{
		cout << "Graphics card not supported; could not find a suitable fragment shader profile.\n";
	}

	// create actual Cg program from file; last two parameters are entry point and arguments
	vertexProgram = cgCreateProgramFromFile(context, CG_SOURCE, vertexShader, vertexProfile, "main", 0);
	if(!vertexProgram)
	{
		cout << "Could not read and creaet vertex shader program.\n";
	}
    cgGLLoadProgram(vertexProgram);

	fragmentProgram = cgCreateProgramFromFile(context, CG_SOURCE, fragmentShader, fragmentProfile, "main", 0);
	if(!fragmentProgram)
	{
		cout << "Could not read and creaet fragment shader program.\n";
	}
	cgGLLoadProgram(fragmentProgram);
	

    /*cgGLBindProgram(vertexProgram);
    cgGLEnableProfile(vertexProfile);
	cgGLBindProgram(fragmentProgram);
    cgGLEnableProfile(fragmentProfile);
	*/

}

void InitializeExtensions()
{
    if (!glutExtensionSupported("GL_ARB_multitexture")) {
        cerr << "initExtensions(): 'GL_ARB_multitexture' extension not supported" << endl;
        exit(1);
    }

    if (!glutExtensionSupported("GL_EXT_texture_env_combine")) {
        cerr << "initExtensions(): 'GL_EXT_texture_env_combine' extension not supported" << endl;
        exit(1);
    }

    if (!glutExtensionSupported("GL_EXT_secondary_color")) {
        cerr << "initExtensions(): 'GL_EXT_secondary_color' extension not supported" << endl;
        exit(1);
    }

    int maxTexelUnits;
    glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &maxTexelUnits);

    if (maxTexelUnits <= 1) {
        cerr << "initExtensions(): the hardware does not support more than 1 (multi-)texture unit" << endl;
        exit(1);
    }

    glMultiTexCoord1fARB = (PFNGLMULTITEXCOORD1FARBPROC) wglGetProcAddress("glMultiTexCoord1fARB");
    glMultiTexCoord2fARB = (PFNGLMULTITEXCOORD2FARBPROC) wglGetProcAddress("glMultiTexCoord2fARB");
    glMultiTexCoord3fARB = (PFNGLMULTITEXCOORD3FARBPROC) wglGetProcAddress("glMultiTexCoord3fARB");
    glMultiTexCoord4fARB = (PFNGLMULTITEXCOORD4FARBPROC) wglGetProcAddress("glMultiTexCoord4fARB");

	//new code
	glTexImage3D = (PFNGLTEXIMAGE3DPROC) wglGetProcAddress("glTexImage3D");

    glActiveTextureARB = (PFNGLACTIVETEXTUREARBPROC) wglGetProcAddress("glActiveTextureARB");
    glClientActiveTextureARB = (PFNGLCLIENTACTIVETEXTUREARBPROC) wglGetProcAddress("glClientActiveTextureARB");

    glSecondaryColor3fEXT = (PFNGLSECONDARYCOLOR3FEXTPROC) wglGetProcAddress("glSecondaryColor3fEXT");
    glSecondaryColor3fvEXT = (PFNGLSECONDARYCOLOR3FVEXTPROC) wglGetProcAddress("glSecondaryColor3fvEXT");
}
