// TGALoader.cpp: Implementierung der Klasse TGALoader
//
///////////////////////////////////////////////////////////////////////


#include <windows.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <corona.h>

#include "TGALoader.h"
#include "Vector4.h"

using namespace corona;

TGALoader::TGALoader()
{
  image = 0;
  type = 0;
  ID = -1;
  imageWidth = 0;
  imageHeight = 0;
}


TGALoader::~TGALoader()
{
	clear();
}



bool TGALoader::LoadTGA(char* file)
{
	FILE *pfile;
	unsigned char tempColor;
	unsigned char bitCount;
	int colorMode;
	long tgaSize;
	unsigned char unCompressHeader[12] = {0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	unsigned char tgaHeader[12];
	unsigned char header[6];

	// open the file
	if(!file) return false;
	pfile = fopen(file, "rb");
	if(!pfile) return false;

	// read the header
	fread(tgaHeader, 1, sizeof(tgaHeader), pfile);

	// we only want to read uncompressed tga's; compare the header
	// with data that would be in a uncompressed header
	if(memcmp(unCompressHeader, tgaHeader, sizeof(unCompressHeader)) != 0)
	{
		fclose(pfile);
		return false;
	}

	// read image info
	fread(header, 1, sizeof(header), pfile);

	// calculate and save width and height
	imageWidth = header[1] * 256 + header[0];
	imageHeight = header[3] * 256 + header[2];

	bitCount = header[4];

	// calculate the color mode and image size.
	colorMode = bitCount / 8;
	tgaSize = imageWidth * imageHeight * colorMode;

	// allocate data for the image and load it
	image = new unsigned char[sizeof(unsigned char) * tgaSize];
	fread(image, sizeof(unsigned char), tgaSize, pfile);

	// convert from BGR to RGB format
	for(long index = 0; index < tgaSize; index += colorMode)
	{
		tempColor = image[index];
		image[index] = image[index + 2];
		image[index + 2] = tempColor;
	}

	//close the file
	fclose(pfile);

	if(colorMode == 3) type = 3;
	else type = 4;

	return true;
}



bool TGALoader::WriteTGA(char *file, int width, int height, unsigned char *outImage)
{
   FILE *pFile = 0;
   unsigned char tgaHeader[12] = {0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0};
   unsigned char header[6];
   unsigned char bits = 0;
   int colorMode = 0;
   unsigned char tempColors = 0;

   // Open file for output.
   pFile = fopen(file, "wb");

   // Check if the file opened or not.
   if(!pFile) { fclose(pFile); return false; }

   // Set the color mode, and the bit depth.
   colorMode = 3;
   bits = 24;

   // Save the width and height.
 	header[0] = width % 256;
 	header[1] = width / 256;
 	header[2] = height % 256;
	header[3] = height / 256;
	header[4] = bits;
	header[5] = 0;

   // Write the headers to the top of the file.
   fwrite(tgaHeader, sizeof(tgaHeader), 1, pFile);
   fwrite(header, sizeof(header), 1, pFile);

   // Now switch image from RGB to BGR.
   for(int index = 0; index < width * height * colorMode; index += colorMode)
      {
         tempColors = outImage[index];
         outImage[index] = outImage[index + 2];
         outImage[index + 2] = tempColors;
      }

   // Finally write the image.
   fwrite(outImage, width * height * colorMode, 1, pFile);

   // close the file.
   fclose(pFile);
   return true;
}

void TGALoader::clear()
{
	if(image)
    {
		delete[] image;
        image = NULL;
    }
}

void TGALoader::LoadDetail(char* file)
{
	Image* detailTexture = FlipImage(OpenImage(file,PF_R8G8B8),CA_X);
	Array3<unsigned char> detailData(detailTexture->getWidth(), detailTexture->getHeight(),3,(unsigned char*)detailTexture->getPixels());

	glPushAttrib(GL_TEXTURE_BIT);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    // Generate the texture
    glGenTextures(1, &detailHandle);
    glBindTexture(GL_TEXTURE_2D, detailHandle);


    gluBuild2DMipmaps(GL_TEXTURE_2D,
        GL_RGB8,
        detailData.getWidth(),
        detailData.getHeight(),
        GL_RGB,
        GL_UNSIGNED_BYTE,
        detailData.getData());

    glPopAttrib();
}

void TGALoader::LoadBumpmap(char* file)
{
	Image* bumpmapTexture = FlipImage(OpenImage(file,PF_R8G8B8),CA_X);
	Array3<unsigned char> bumpmapData(bumpmapTexture->getWidth(), bumpmapTexture->getHeight(),3,(unsigned char*)bumpmapTexture->getPixels());
	glPushAttrib(GL_TEXTURE_BIT);

    // Scale luminance 50% because blending will double the intensity:
    // glBlend(GL_DST_COLOR, GL_SRC_COLOR)
    glPixelTransferf(GL_RED_SCALE, 0.5f);
    glPixelTransferf(GL_GREEN_SCALE, 0.5f);
    glPixelTransferf(GL_BLUE_SCALE, 0.5f);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

    // Generate the bumpmap
    glGenTextures(1, &bumpmapHandle);
    glBindTexture(GL_TEXTURE_2D, bumpmapHandle);

    gluBuild2DMipmaps(GL_TEXTURE_2D,
        GL_RGB8,
        bumpmapData.getWidth(),
        bumpmapData.getHeight(),
        GL_RGB,
        GL_UNSIGNED_BYTE,
        bumpmapData.getData());

    // Generate the inverted bumpmap
    glGenTextures(1, &invertedBumpmapHandle);
    glBindTexture(GL_TEXTURE_2D, invertedBumpmapHandle);

    // NOTE: luminance still at 50%
    Array3<unsigned char> invertedBumpmapData(bumpmapData.getWidth(), bumpmapData.getHeight(), 3);
    for (int i = 0; i < bumpmapData.getWidth(); i++) {
        for (int j = 0; j < bumpmapData.getHeight(); j++) {
            invertedBumpmapData(i, j, 0) = 255 - bumpmapData(i, j, 0);
            invertedBumpmapData(i, j, 1) = 255 - bumpmapData(i, j, 1);
            invertedBumpmapData(i, j, 2) = 255 - bumpmapData(i, j, 2);
        }
    }

    gluBuild2DMipmaps(GL_TEXTURE_2D,
        GL_RGB8,
        invertedBumpmapData.getWidth(),
        invertedBumpmapData.getHeight(),
        GL_RGB,
        GL_UNSIGNED_BYTE,
        invertedBumpmapData.getData());

    // Scale luminance back to 100%
    glPixelTransferf(GL_RED_SCALE, 1.0f);
    glPixelTransferf(GL_GREEN_SCALE, 1.0f);
    glPixelTransferf(GL_BLUE_SCALE, 1.0f);

    glPopAttrib();
}
	   
void TGALoader::LoadNormalMap(char* file)
{
	Image* bumpmapTexture = FlipImage(OpenImage(file,PF_R8G8B8),CA_X);
	Array3<unsigned char> bumpmapData(bumpmapTexture->getWidth(), bumpmapTexture->getHeight(),3,(unsigned char*)bumpmapTexture->getPixels());

	float strength = 0.01f;
    
    glPushAttrib(GL_TEXTURE_BIT);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    // Generate the normalmap
    glGenTextures(1, &normalmapHandle);
    glBindTexture(GL_TEXTURE_2D, normalmapHandle);

    // Compute the center of the normalmapData (skip the edges
    // since we can't readily compute slopes at the edge).
    Array3<unsigned char> normalmapData(bumpmapData.getWidth(), bumpmapData.getHeight(), 3);
    for (int i = 1; i < bumpmapData.getWidth() - 1; i++) {
        for (int j = 1; j < bumpmapData.getHeight() - 1; j++) {
            //Vector s(1, 0, strength * (bumpmapData(i + 1, j, 0) - bumpmapData(i - 1, j, 0)));
			Vector4 s(1.0, 0.0, strength*(bumpmapData(i+1,j,0.0)-bumpmapData(i-1,j,0.0)));
            //Vector t(0, 1, strength * (bumpmapData(i, j + 1, 0) - bumpmapData(i, j - 1, 0)));
			Vector4 t(0.0, 1.0, strength*(bumpmapData(i,j+1,0.0)-bumpmapData(i,j-1,0.0)));
			//Vector n = s.cross(t).normalize().rangeCompress(255);
			Vector4 n;
			n.CrossProduct(s,t);
			n.Normalize();
			n.RangeCompress(255);

            //normalmapData(i, j, 0) = (GLbyte) n[Vector::X];
            //normalmapData(i, j, 1) = (GLbyte) n[Vector::Y];
            //normalmapData(i, j, 2) = (GLbyte) n[Vector::Z];
			normalmapData(i, j, 0) = (GLbyte) n.x;
            normalmapData(i, j, 1) = (GLbyte) n.y;
            normalmapData(i, j, 2) = (GLbyte) n.z;
        }
    }

    // Fill in the top and bottom edge
    for (i = 0; i < bumpmapData.getWidth(); i++) {
        normalmapData(i, 0, 0) = normalmapData(i, 1, 0);
        normalmapData(i, 0, 1) = normalmapData(i, 1, 1);
        normalmapData(i, 0, 2) = normalmapData(i, 1, 2);

        int last = normalmapData.getHeight() - 1;
        normalmapData(i, last, 0) = normalmapData(i, last - 1, 0);
        normalmapData(i, last, 1) = normalmapData(i, last - 1, 1);
        normalmapData(i, last, 2) = normalmapData(i, last - 1, 2);
    }

    // Fill in the left and right edge
    for (i = 0; i < bumpmapData.getHeight(); i++) {
        normalmapData(0, i, 0) = normalmapData(1, i, 0);
        normalmapData(0, i, 1) = normalmapData(1, i, 1);
        normalmapData(0, i, 2) = normalmapData(1, i, 2);

        int last = normalmapData.getWidth() - 1;
        normalmapData(last, i, 0) = normalmapData(last - 1, i, 0);
        normalmapData(last, i, 1) = normalmapData(last - 1, i, 1);
        normalmapData(last, i, 2) = normalmapData(last - 1, i, 2);
    }

    gluBuild2DMipmaps(GL_TEXTURE_2D,
        GL_RGB8,
        normalmapData.getWidth(),
        normalmapData.getHeight(),
        GL_RGB,
        GL_UNSIGNED_BYTE,
        normalmapData.getData());

    glPopAttrib();
}